@@ -6,8 +6,9 @@ Multi-class / multi-label utility function
 ==========================================
 
 """
-from __future__ import division
+from collections.abc import Sequence
 from itertools import chain
+import warnings
 
 from scipy.sparse import issparse
 from scipy.sparse.base import spmatrix
@@ -16,9 +17,7 @@ from scipy.sparse import lil_matrix
 
 import numpy as np
 
-from ..externals.six import string_types
-from ..utils.fixes import _Sequence as Sequence
-from .validation import check_array
+from .validation import check_array, _assert_all_finite
 
 
 def _unique_multiclass(y):
@@ -29,7 +28,9 @@ def _unique_multiclass(y):
 
 
 def _unique_indicator(y):
-    return np.arange(check_array(y, ['csr', 'csc', 'coo']).shape[1])
+    return np.arange(
+        check_array(y, accept_sparse=['csr', 'csc', 'coo']).shape[1]
+    )
 
 
 _FN_UNIQUE_LABELS = {
@@ -40,7 +41,7 @@ _FN_UNIQUE_LABELS = {
 
 
 def unique_labels(*ys):
-    """Extract an ordered array of unique labels
+    """Extract an ordered array of unique labels.
 
     We don't allow:
         - mix of multilabel and multiclass (single label) targets
@@ -57,7 +58,7 @@ def unique_labels(*ys):
 
     Returns
     -------
-    out : numpy array of shape [n_unique_labels]
+    out : ndarray of shape (n_unique_labels,)
         An ordered array of unique labels.
 
     Examples
@@ -75,8 +76,8 @@ def unique_labels(*ys):
     # Check that we don't mix label format
 
     ys_types = set(type_of_target(x) for x in ys)
-    if ys_types == set(["binary", "multiclass"]):
-        ys_types = set(["multiclass"])
+    if ys_types == {"binary", "multiclass"}:
+        ys_types = {"multiclass"}
 
     if len(ys_types) > 1:
         raise ValueError("Mix type of y not allowed, got types %s" % ys_types)
@@ -85,7 +86,8 @@ def unique_labels(*ys):
 
     # Check consistency for the indicator format
     if (label_type == "multilabel-indicator" and
-            len(set(check_array(y, ['csr', 'csc', 'coo']).shape[1]
+            len(set(check_array(y,
+                                accept_sparse=['csr', 'csc', 'coo']).shape[1]
                     for y in ys)) > 1):
         raise ValueError("Multi-label binary indicator input with "
                          "different numbers of labels")
@@ -98,7 +100,7 @@ def unique_labels(*ys):
     ys_labels = set(chain.from_iterable(_unique_labels(y) for y in ys))
 
     # Check that we don't mix string type with number type
-    if (len(set(isinstance(label, string_types) for label in ys_labels)) > 1):
+    if (len(set(isinstance(label, str) for label in ys_labels)) > 1):
         raise ValueError("Mix of label input types (string and number)")
 
     return np.array(sorted(ys_labels))
@@ -113,12 +115,12 @@ def is_multilabel(y):
 
     Parameters
     ----------
-    y : numpy array of shape [n_samples]
+    y : ndarray of shape (n_samples,)
         Target values.
 
     Returns
     -------
-    out : bool,
+    out : bool
         Return ``True``, if ``y`` is in a multilabel format, else ```False``.
 
     Examples
@@ -136,8 +138,18 @@ def is_multilabel(y):
     >>> is_multilabel(np.array([[1, 0, 0]]))
     True
     """
-    if hasattr(y, '__array__'):
-        y = np.asarray(y)
+    if hasattr(y, '__array__') or isinstance(y, Sequence):
+        # DeprecationWarning will be replaced by ValueError, see NEP 34
+        # https://numpy.org/neps/nep-0034-infer-dtype-is-object.html
+        with warnings.catch_warnings():
+            warnings.simplefilter('error', np.VisibleDeprecationWarning)
+            try:
+                y = np.asarray(y)
+            except np.VisibleDeprecationWarning:
+                # dtype=object should be provided explicitly for ragged arrays,
+                # see NEP 34
+                y = np.array(y, dtype=object)
+
     if not (hasattr(y, "shape") and y.ndim == 2 and y.shape[1] > 1):
         return False
 
@@ -189,7 +201,7 @@ def type_of_target(y):
 
     Returns
     -------
-    target_type : string
+    target_type : str
         One of:
 
         * 'continuous': `y` is an array-like of floats that are not all
@@ -229,46 +241,52 @@ def type_of_target(y):
     >>> type_of_target(np.array([[1, 2], [3, 1]]))
     'multiclass-multioutput'
     >>> type_of_target([[1, 2]])
-    'multiclass-multioutput'
+    'multilabel-indicator'
     >>> type_of_target(np.array([[1.5, 2.0], [3.0, 1.6]]))
     'continuous-multioutput'
     >>> type_of_target(np.array([[0, 1], [1, 1]]))
     'multilabel-indicator'
     """
     valid = ((isinstance(y, (Sequence, spmatrix)) or hasattr(y, '__array__'))
-             and not isinstance(y, string_types))
+             and not isinstance(y, str))
 
     if not valid:
         raise ValueError('Expected array-like (array or non-string sequence), '
                          'got %r' % y)
 
-    sparseseries = (y.__class__.__name__ == 'SparseSeries')
-    if sparseseries:
-        raise ValueError("y cannot be class 'SparseSeries'.")
+    sparse_pandas = (y.__class__.__name__ in ['SparseSeries', 'SparseArray'])
+    if sparse_pandas:
+        raise ValueError("y cannot be class 'SparseSeries' or 'SparseArray'")
 
     if is_multilabel(y):
         return 'multilabel-indicator'
 
-    try:
-        y = np.asarray(y)
-    except ValueError:
-        # Known to fail in numpy 1.3 for array of arrays
-        return 'unknown'
+    # DeprecationWarning will be replaced by ValueError, see NEP 34
+    # https://numpy.org/neps/nep-0034-infer-dtype-is-object.html
+    with warnings.catch_warnings():
+        warnings.simplefilter('error', np.VisibleDeprecationWarning)
+        try:
+            y = np.asarray(y)
+        except np.VisibleDeprecationWarning:
+            # dtype=object should be provided explicitly for ragged arrays,
+            # see NEP 34
+            y = np.asarray(y, dtype=object)
 
     # The old sequence of sequences format
     try:
         if (not hasattr(y[0], '__array__') and isinstance(y[0], Sequence)
-                and not isinstance(y[0], string_types)):
+                and not isinstance(y[0], str)):
             raise ValueError('You appear to be using a legacy multi-label data'
                              ' representation. Sequence of sequences are no'
                              ' longer supported; use a binary array or sparse'
-                             ' matrix instead.')
+                             ' matrix instead - the MultiLabelBinarizer'
+                             ' transformer can convert to this format.')
     except IndexError:
         pass
 
     # Invalid inputs
     if y.ndim > 2 or (y.dtype == object and len(y) and
-                      not isinstance(y.flat[0], string_types)):
+                      not isinstance(y.flat[0], str)):
         return 'unknown'  # [[[1, 2]]] or [obj_1] and not ["label_1"]
 
     if y.ndim == 2 and y.shape[1] == 0:
@@ -282,6 +300,7 @@ def type_of_target(y):
     # check float and contains non-integer float values
     if y.dtype.kind == 'f' and np.any(y != y.astype(int)):
         # [.1, .2, 3] or [[.1, .2, 3]] or [[1., .2]] and not [1., 2., 3.]
+        _assert_all_finite(y)
         return 'continuous' + suffix
 
     if (len(np.unique(y)) > 2) or (y.ndim >= 2 and len(y[0]) > 1):
@@ -291,7 +310,7 @@ def type_of_target(y):
 
 
 def _check_partial_fit_first_call(clf, classes=None):
-    """Private helper function for factorizing common classes param logic
+    """Private helper function for factorizing common classes param logic.
 
     Estimators that implement the ``partial_fit`` API need to be provided with
     the list of possible classes at the first call to partial_fit.
@@ -326,25 +345,25 @@ def _check_partial_fit_first_call(clf, classes=None):
 
 
 def class_distribution(y, sample_weight=None):
-    """Compute class priors from multioutput-multiclass target data
+    """Compute class priors from multioutput-multiclass target data.
 
     Parameters
     ----------
-    y : array like or sparse matrix of size (n_samples, n_outputs)
+    y : {array-like, sparse matrix} of size (n_samples, n_outputs)
         The labels for each example.
 
-    sample_weight : array-like of shape = (n_samples,), optional
+    sample_weight : array-like of shape (n_samples,), default=None
         Sample weights.
 
     Returns
     -------
-    classes : list of size n_outputs of arrays of size (n_classes,)
+    classes : list of size n_outputs of ndarray of size (n_classes,)
         List of classes for each column.
 
-    n_classes : list of integers of size n_outputs
-        Number of classes in each column
+    n_classes : list of int of size n_outputs
+        Number of classes in each column.
 
-    class_prior : list of size n_outputs of arrays of size (n_classes,)
+    class_prior : list of size n_outputs of ndarray of size (n_classes,)
         Class distribution of each column.
 
     """
@@ -353,6 +372,8 @@ def class_distribution(y, sample_weight=None):
     class_prior = []
 
     n_samples, n_outputs = y.shape
+    if sample_weight is not None:
+        sample_weight = np.asarray(sample_weight)
 
     if issparse(y):
         y = y.tocsc()
@@ -362,7 +383,7 @@ def class_distribution(y, sample_weight=None):
             col_nonzero = y.indices[y.indptr[k]:y.indptr[k + 1]]
             # separate sample weights for zero and non-zero elements
             if sample_weight is not None:
-                nz_samp_weight = np.asarray(sample_weight)[col_nonzero]
+                nz_samp_weight = sample_weight[col_nonzero]
                 zeros_samp_weight_sum = (np.sum(sample_weight) -
                                          np.sum(nz_samp_weight))
             else:
@@ -407,16 +428,16 @@ def _ovr_decision_function(predictions, confidences, n_classes):
 
     Parameters
     ----------
-    predictions : array-like, shape (n_samples, n_classifiers)
+    predictions : array-like of shape (n_samples, n_classifiers)
         Predicted classes for each binary classifier.
 
-    confidences : array-like, shape (n_samples, n_classifiers)
+    confidences : array-like of shape (n_samples, n_classifiers)
         Decision functions or predicted probabilities for positive class
         for each binary classifier.
 
     n_classes : int
         Number of classes. n_classifiers must be
-        ``n_classes * (n_classes - 1 ) / 2``
+        ``n_classes * (n_classes - 1 ) / 2``.
     """
     n_samples = predictions.shape[0]
     votes = np.zeros((n_samples, n_classes))
@@ -431,17 +452,13 @@ def _ovr_decision_function(predictions, confidences, n_classes):
             votes[predictions[:, k] == 1, j] += 1
             k += 1
 
-    max_confidences = sum_of_confidences.max()
-    min_confidences = sum_of_confidences.min()
-
-    if max_confidences == min_confidences:
-        return votes
-
-    # Scale the sum_of_confidences to (-0.5, 0.5) and add it with votes.
+    # Monotonically transform the sum_of_confidences to (-1/3, 1/3)
+    # and add it with votes. The monotonic transformation  is
+    # f: x -> x / (3 * (|x| + 1)), it uses 1/3 instead of 1/2
+    # to ensure that we won't reach the limits and change vote order.
     # The motivation is to use confidence levels as a way to break ties in
     # the votes without switching any decision made based on a difference
     # of 1 vote.
-    eps = np.finfo(sum_of_confidences.dtype).eps
-    max_abs_confidence = max(abs(max_confidences), abs(min_confidences))
-    scale = (0.5 - eps) / max_abs_confidence
-    return votes + sum_of_confidences * scale
+    transformed_confidences = (sum_of_confidences /
+                               (3 * (np.abs(sum_of_confidences) + 1)))
+    return votes + transformed_confidences
